/*
    ModbusIP_ESP8266.cpp - Source for Modbus IP ESP8266 Library
    Copyright (C) 2015 Andr� Sarmento Barbosa
*/
#include "ModbusIP_ESP8266.h"

void dd(byte x) { char s[3]; sprintf(s,"%02X",x); Serial.print(" "+String(s)); }

WiFiServer MODBUS_server(MODBUSIP_PORT);

ModbusIP::ModbusIP() { }

void ModbusIP::begin() { MODBUS_server.begin(); }

void ModbusIP::task() {
  
	WiFiClient mcl = MODBUS_server.available();

	int raw_len = 0;
	
    if (mcl) {
		if (mcl.connected()) {
		    for (int x = 0; x < 300; x++) { // Time to have data available
				if (mcl.available()) {
					while (mcl.available() > raw_len) {  //Computes data length
						raw_len = mcl.available();
						delay(1);
					}
					break;
				}
				delay(10);				
			}
		}

           Serial.print("MODBUS "+String(raw_len)+" [");

  byte last_i=0;

		if (raw_len > 7) {
			for (int i=0; i<7; i++)	{
			  _MBAP[i] = mcl.read(); //Get MBAP
        dd(_MBAP[i]);
        last[last_i++]=_MBAP[i];
			}

			_len = _MBAP[4] << 8 | _MBAP[5];
			_len--; // Do not count with last byte from MBAP
			if (_MBAP[2] !=0 || _MBAP[3] !=0) {
          Serial.println(" ] Not a MODBUSIP packet");
			    return;   // Not a MODBUSIP packet
			}
			if (_len > MODBUSIP_MAXFRAME) {
          Serial.println(" ] Length "+String(_len)+" over");
			    return;      //Length is over MODBUSIP_MAXFRAME
			}
			_frame = (byte*) malloc(_len);
			
			raw_len = raw_len - 7;

      Serial.print(" / ");
			for (int i=0; i< raw_len; i++) {
			  _frame[i] = mcl.read(); //Get Modbus PDU
        if(last_i<12) last[last_i++]=_MBAP[i];
        dd(_frame[i]);
			}
      Serial.print(" ] ");

      while(last_i<12) last[last_i++]=0xAA;
      
			this->receivePDU(_frame);
			mcl.flush();

      // Serial.print(String(_reply != MB_REPLY_OFF ? "REPLY: ":"NO_REPLY"));

			if (_reply != MB_REPLY_OFF) {
			    //MBAP
				_MBAP[4] = (_len+1) >> 8;     //_len+1 for last byte from MBAP
				_MBAP[5] = (_len+1) & 0x00FF;
				
				size_t send_len = (unsigned int)_len + 7;
				uint8_t sbuf[send_len];
				
				for (int i=0; i<7; i++)	    sbuf[i] = _MBAP[i];
				for (int i=0; i<_len; i++)	sbuf[i+7] = _frame[i];

        Serial.print(" -> { "); for (int i=0; i<send_len; i++) dd(sbuf[i]); Serial.print(" }");

				mcl.write(sbuf, send_len);
        
			}

      Serial.print("\n");

			mcl.stop();
			free(_frame);
			_len = 0;
      // Serial.println(" END SESSION");
		}
	}
}
